import Vue from 'vue'
import Router from 'vue-router'

Vue.use(Router)

/* Layout */
import Layout from '@/layout'

/**
 * Note: sub-menu only appear when route children.length >= 1
 * Detail see: https://panjiachen.github.io/vue-element-admin-site/guide/essentials/router-and-nav.html
 *
 * hidden: true                   if set true, item will not show in the sidebar(default is false)
 * alwaysShow: true               if set true, will always show the root menu
 *                                if not set alwaysShow, when item has more than one children route,
 *                                it will becomes nested mode, otherwise not show the root menu
 * redirect: noRedirect           if set noRedirect will no redirect in the breadcrumb
 * name:'router-name'             the name is used by <keep-alive> (must set!!!)
 * meta : {
    roles: ['admin','editor']    control the page roles (you can set multiple roles)
    title: 'title'               the name show in sidebar and breadcrumb (recommend set)
    icon: 'svg-name'/'el-icon-x' the icon show in the sidebar
    breadcrumb: false            if set false, the item will hidden in breadcrumb(default is true)
    activeMenu: '/example/list'  if set path, the sidebar will highlight the path you set
  }
 */

/**
 * constantRoutes
 * a base page that does not have permission requirements
 * all roles can be accessed
 */
export const constantRoutes = [{
    path: '/login',
    component: () => import('@/views/login/index'),
    hidden: true
  },

  {
    path: '/404',
    component: () => import('@/views/404'),
    hidden: true
  },

  {
    path: '/',
    component: Layout,
    redirect: '/dashboard',
    children: [{
      path: 'dashboard',
      name: 'Dashboard',
      component: () => import('@/views/dashboard/index'),
      meta: {
        title: '数据看板',
        icon: 'dashboard'
      }
    }]
  },
  {
    path: '/user',
    component: Layout,
    props: true,
    meta: {
      roles: ['admin'],
      title: "用户管理",
      icon: 'user',
    },
    children: [{
      path: 'index',
      name: 'user',
      component: () => import('@/views/user/index'),
      meta: {
        title: '用户管理',
        icon: 'user'
      }
    }]
  },
  // {
  //   path: '/role',
  //   component: Layout,
  //   children: [
  //     {
  //       path: 'index',
  //       name: 'role',
  //       component: () => import('@/views/role/index'),
  //       meta: { title: '角色管理', icon: 'role' }
  //     }
  //   ]
  // },
  {
    path: '/sys-param',
    component: Layout,
    meta: {
      roles: ['admin'],
      title: "系统参数",
      icon: 'sys-param',
    },
    children: [{
      path: 'index',
      name: 'sys-param',
      component: () => import('@/views/sys-param/index'),
      meta: {
        title: '系统参数',
        icon: 'sys-param'
      }
    }]
  },

  {
    path: '/autoplay',
    component: Layout,
    meta: {
      title: '任务管理',
      icon: 'autoplay'
    },
    children: [{
      path: 'index',
      name: 'autoplay',
      component: () => import('@/views/autoplay/index'),
      meta: {
        title: '任务管理',
        icon: 'autoplay'
      }
    }]
  },

  {
    path: '/airdrop',
    component: Layout,
    children: [{
      path: 'index',
      name: 'airdrop',
      component: () => import('@/views/airdrop/index'),
      meta: {
        title: '项目管理',
        icon: 'airdrop'
      }
    }]
  },
  // {
  //   path: '/strategy',
  //   component: Layout,
  //   children: [{
  //     path: 'index',
  //     name: 'strategy',
  //     component: () => import('@/views/strategy/index'),
  //     meta: {
  //       title: '策略管理',
  //       icon: 'strategy'
  //     }
  //   }]
  // },
  {
    path: '/address',
    component: Layout,
    children: [{
      path: 'index',
      name: 'address',
      component: () => import('@/views/address/index'),
      meta: {
        title: '地址管理',
        icon: 'address'
      }
    }]
  },

  // 404 page must be placed at the end !!!
  {
    path: '*',
    redirect: '/404',
    hidden: true
  }
]

const createRouter = () => new Router({
  // mode: 'history', // require service support
  scrollBehavior: () => ({
    y: 0
  }),
  routes: constantRoutes
})

const router = createRouter()

// Detail see: https://github.com/vuejs/vue-router/issues/1234#issuecomment-357941465
export function resetRouter() {
  const newRouter = createRouter()
  router.matcher = newRouter.matcher // reset router
}

export default router
