package com.ichaoj.ams.common.util;

import cn.hutool.core.io.IoUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.SecureUtil;
import cn.hutool.http.Header;
import com.ichaoj.web.context.SuperWhaleContext;
import io.seata.common.util.IOUtil;
import lombok.SneakyThrows;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.io.IOUtils;
import org.springframework.http.MediaType;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.commons.CommonsMultipartFile;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.List;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

/**
 * @author cjwen
 */
@Slf4j
public class FileUtils {
    @SneakyThrows
    public static MultipartFile convertFileToMultipartFile(File file) throws IOException {
        FileItem item = new DiskFileItemFactory().createItem(
                "file",
                MediaType.MULTIPART_FORM_DATA_VALUE,
                true,
                file.getName()
        );
        FileInputStream inputStream = new FileInputStream(file);
        OutputStream outputStream = item.getOutputStream();
        IoUtil.copy(inputStream, outputStream);
        return new CommonsMultipartFile(item);
    }

    /**
     * 将文件夹压缩成zip文件下载
     *
     * @param dirPath 文件夹路径
     * @param zipName zip文件名
     */
    public static void downloadDirAsZip(String dirPath, String zipName) throws IOException {
        // 获取文件夹下所有文件
        List<File> files = cn.hutool.core.io.FileUtil.loopFiles(dirPath);

        // 创建zip输出流
        ByteArrayOutputStream out = new ByteArrayOutputStream();
        ZipOutputStream zipOut = new ZipOutputStream(out);

        // 将每个文件添加到zip输出流
        for (File file : files) {
            zipOut.putNextEntry(new ZipEntry(file.getName()));
            FileInputStream in = new FileInputStream(file);

            IoUtil.copy(in, zipOut);

            IOUtil.close(in);
            zipOut.closeEntry();
        }

        // 关闭zip输出流
        IOUtil.close(zipOut);

        // 获取zip文件byte数组
        byte[] data = out.toByteArray();

        HttpServletResponse response = SuperWhaleContext.getContext(HttpServletResponse.class);

        // 设置response头
        genZip(response, data, zipName);
        IoUtil.close(out);
    }

    /**
     * 生成zip
     */
    private static void genZip(HttpServletResponse response, byte[] data, String zipName) throws IOException {
        response.reset();
        response.setHeader("Content-Disposition", StrUtil.format("attachment; filename=\"{}.zip\"", SecureUtil.sha256(zipName)));
        response.addHeader("Content-Length", "" + data.length);
        response.setHeader(Header.CONTENT_TYPE.getValue(), "multipart/form-data");
        response.setContentType("application/octet-stream; charset=UTF-8");
        IOUtils.write(data, response.getOutputStream());
    }

}