package com.ichaoj.ams.common.util;

import com.alibaba.fastjson.JSONObject;
import io.swagger.v3.oas.annotations.media.Schema;
import io.swagger.v3.oas.annotations.tags.Tag;
import lombok.Getter;
import lombok.Setter;


/**
 * @author ben
 */
@Setter
@Getter
@Tag(name = "响应参数")
public class MessageResult<T> {
    @Schema(title = "状态码(例: 0:成功, 500:系统错误)")
    private int code;
    @Schema(title = "描述信息")
    private String message;
    @Schema(title = "返回数据")
    private T data;

    public MessageResult() {
    }

    public MessageResult(int code, String msg) {
        this.code = code;
        this.message = msg;
    }

    /**
     * use the 'replacements' to replace and format 'msg' <br>
     * see @java.lang.String#format(java.lang.String, java.lang.Object...)
     *
     * @param code
     * @param msg
     * @param replacements
     */
    public MessageResult(int code, String msg, T... replacements) {
        this.code = code;
        this.message = String.format(msg, replacements);
    }

    public MessageResult(int code, String msg, T data) {
        this.code = code;
        this.message = msg;
        this.data = data;
    }

    public static MessageResult success() {
        return new MessageResult(0, "SUCCESS");
    }

    public static MessageResult success(String msg, Object obj) {
        MessageResult mr = new MessageResult(0, msg);
        mr.setData(obj);
        return mr;
    }

    public static MessageResult success(Object data) {
        return new MessageResult(0, "SUCCESS", data);
    }

    public static MessageResult error(int code, String msg) {
        return new MessageResult(code, msg);
    }

    public static MessageResult error(String msg) {
        return new MessageResult(500, msg);
    }

    @Override
    public String toString() {
        return JSONObject.toJSONString(this);
    }
}
