package com.ichaoj.ams.common.util;

import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.servlet.http.HttpServletResponse;
import java.io.*;
import java.util.zip.ZipEntry;
import java.util.zip.ZipOutputStream;

import static com.ichaoj.ams.constant.AmsConstant.WALLET_FILE_PATH;

/**
 * @author cjwen
 */
@Slf4j
public class ZipUtil {
    /**
     * 创建并下载zip文件
     *
     * @param groupName 地址组名
     * @param response  响应结果
     * @param userId    用户id
     */
    public static void createZip(String groupName, HttpServletResponse response, String userId) {
        InputStream inputStream = null;
        OutputStream outputStream = null;
        File zipFile = null;
        try {


            // 将json_files目录下的所有文件打包成zip
            zipFile = new File(WalletUtil.getEncryptPwd(groupName) + ".zip");
            ZipOutputStream zos = new ZipOutputStream(new FileOutputStream(zipFile));

            File fileDir = new File(WALLET_FILE_PATH + userId + File.separator + WalletUtil.getEncryptPwd(groupName));
            File[] files = fileDir.listFiles();
            for (File file : files) {
                ZipEntry entry = new ZipEntry(file.getName());
                zos.putNextEntry(entry);
                FileInputStream fis = new FileInputStream(file);
                byte[] buffer = new byte[1024];
                int len;
                while ((len = fis.read(buffer)) > 0) {
                    zos.write(buffer, 0, len);
                }
                fis.close();
            }
            zos.close();

            // 设置响应头,使浏览器下载该zip文件
            response.addHeader("Content-Disposition", "attachment;filename=" + WalletUtil.getEncryptPwd(groupName) + ".zip");
            inputStream = new FileInputStream(zipFile);
            byte[] buffer = new byte[1024];
            outputStream = response.getOutputStream();
            int len;
            while ((len = inputStream.read(buffer)) > 0) {
                outputStream.write(buffer, 0, len);
            }
        } catch (IOException ex) {
            log.error("batchKeystore 方法报错: {}", ex.getMessage());
            ex.printStackTrace();
        } finally {
            try {
                if (inputStream != null) {
                    inputStream.close();
                }
                if (outputStream != null) {
                    outputStream.close();
                }
                if (zipFile != null) {
                    zipFile.delete();
                }
            } catch (Exception e) {
                log.info(e.getMessage());
            }
        }
    }
}
