package com.ichaoj.ams.script;


import com.ichaoj.ams.script.model.AirdropParam;
import com.ichaoj.ams.script.model.AirdropWallet;
import com.ichaoj.ams.script.util.Web3Util;
import org.web3j.crypto.Credentials;
import org.web3j.crypto.RawTransaction;
import org.web3j.crypto.TransactionEncoder;
import org.web3j.protocol.Web3j;
import org.web3j.protocol.core.DefaultBlockParameterName;
import org.web3j.protocol.core.Response;
import org.web3j.protocol.core.methods.response.EthSendTransaction;
import org.web3j.utils.Numeric;

import java.math.BigInteger;
import java.util.Map;

/**
 * 脚本接口
 *
 * @author wren
 */
public abstract class IScript {


    /**
     * 运行脚本
     *
     * @param params 参数
     * @return txId
     */
    public abstract SResult run(Map<String, AirdropParam> params, AirdropWallet airdropWallet);

    protected SResult sendTx(AirdropWallet airdropWallet, Web3j web3j, RawTransaction etherTransaction) throws java.io.IOException {
        Credentials credentials = Web3Util.getCredentials(airdropWallet);
        byte[] signMessage = TransactionEncoder.signMessage(etherTransaction, credentials);
        String hexValue = Numeric.toHexString(signMessage);
        EthSendTransaction ethSendTransaction = web3j.ethSendRawTransaction(hexValue).send();
        Response.Error error = ethSendTransaction.getError();
        if (error != null) {
            throw new RuntimeException(error.getMessage());
        }
        org.web3j.protocol.core.methods.response.Transaction transaction = web3j
                .ethGetTransactionByHash(ethSendTransaction.getTransactionHash())
                .send()
                .getTransaction()
                .get();
        BigInteger gasAmount = transaction.getGas().multiply(transaction.getGasPrice());
        SResult sResult = new SResult();
        BigInteger balance = web3j.ethGetBalance(airdropWallet.getAddress(), DefaultBlockParameterName.LATEST).send().getBalance();
        sResult.setGas(gasAmount.toString());
        sResult.setTxId(transaction.getHash());
        sResult.setCurrentBalance(balance.toString());
        return sResult;
    }



}
