package com.ichaoj.ams.script.zksync.era;

import cn.hutool.core.collection.ListUtil;
import cn.hutool.core.util.IdUtil;
import com.ichaoj.ams.script.IScript;
import com.ichaoj.ams.script.SResult;
import com.ichaoj.ams.script.annotation.Script;
import com.ichaoj.ams.script.annotation.ScriptParam;
import com.ichaoj.ams.script.model.AirdropParam;
import com.ichaoj.ams.script.model.AirdropWallet;
import com.ichaoj.ams.script.util.Web3Util;
import lombok.SneakyThrows;
import org.jetbrains.annotations.NotNull;
import org.web3j.abi.FunctionEncoder;
import org.web3j.abi.TypeReference;
import org.web3j.abi.datatypes.*;
import org.web3j.abi.datatypes.generated.Bytes32;
import org.web3j.abi.datatypes.generated.Uint256;
import org.web3j.crypto.Credentials;
import org.web3j.crypto.RawTransaction;
import org.web3j.crypto.TransactionEncoder;
import org.web3j.protocol.Web3j;
import org.web3j.protocol.core.DefaultBlockParameterName;
import org.web3j.protocol.core.Response;
import org.web3j.protocol.core.methods.request.Transaction;
import org.web3j.protocol.core.methods.response.EthGetBalance;
import org.web3j.protocol.core.methods.response.EthSendTransaction;
import org.web3j.protocol.http.HttpService;
import org.web3j.utils.Convert;
import org.web3j.utils.Numeric;

import java.io.IOException;
import java.math.BigDecimal;
import java.math.BigInteger;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;

/**
 * @author wrenj
 */
@Script(name = "zksync-cross",
        params = {
                @ScriptParam(name = "amount", note = "跨链金额"),
                @ScriptParam(name = "value", note = "转账value，要求覆盖L2的手续费"),
                @ScriptParam(name = "l2GasLimit", note = "L2 gas limit", defaultValue = "901876"),

        })
public class ZkSyncCrossScript extends IScript {

    @SneakyThrows
    @Override
    public SResult run(Map<String, AirdropParam> params, AirdropWallet airdropWallet) {
        AirdropParam amount = params.get("amount");
        AirdropParam value = params.get("value");
        AirdropParam l2GasLimit = params.get("l2GasLimit");
        String mailBox = "0x32400084c286cf3e17e7b677ea9583e60a000324";
        Web3j web3j = Web3j.build(new HttpService("https://eth-mainnet.g.alchemy.com/v2/zS4UDTUziif2jL82eFGoP_sddA8aVgEV"));
        List<Type> inputParameters = new ArrayList<>();
        //recipient
        inputParameters.add(new Address(airdropWallet.getAddress()));
        //amount
        inputParameters.add(new Uint256(Convert.toWei(amount.getValue(), Convert.Unit.ETHER).toBigInteger()));
        //bytes
        inputParameters.add(DynamicBytes.DEFAULT);
        //l2 gas limit
        inputParameters.add(new Uint256(new BigInteger(l2GasLimit.getValue())));
        //_l2GasPerPub-dataByteLimit
        inputParameters.add(new Uint256(800));
        //data
        inputParameters.add(new DynamicArray<>(DynamicArray.class));
        //_refundRecipient
        inputParameters.add(new Address(airdropWallet.getAddress()));
        Function requestL2Transaction = new Function("requestL2Transaction",
                inputParameters,
                ListUtil.of(new TypeReference<Bytes32>() {
                }));
        BigInteger nonce = Web3Util.getNonce(web3j, airdropWallet.getAddress());
        BigInteger gasPrice = web3j.ethGasPrice().send().getGasPrice();
        String l2Encoder = FunctionEncoder.encode(requestL2Transaction);

        Transaction callTransaction = Transaction.createFunctionCallTransaction(
                airdropWallet.getAddress(),
                nonce,
                gasPrice,
                BigInteger.valueOf(0),
                mailBox,
                Convert.toWei(value.getValue(), Convert.Unit.ETHER).toBigInteger(),
                l2Encoder);
        BigInteger gasLimit = Web3Util.getTransactionGasLimit(web3j, callTransaction);

//        RawTransaction rawTransaction = RawTransaction.createTransaction(nonce,
//                gasPrice,
//                gasLimit,
//                mailBox,
//                Convert.toWei(value.getValue(), Convert.Unit.ETHER).toBigInteger(),
//                l2Encoder);
//
//
//        //签名
//        Credentials credentials = Web3Util.getCredentials(airdropWallet);
//        byte[] signMessage = TransactionEncoder.signMessage(rawTransaction, credentials);
//        String hexValue = Numeric.toHexString(signMessage);
//        //发送交易
//        EthSendTransaction ethSendTransaction = web3j.ethSendRawTransaction(hexValue).send();
//        Response.Error error = ethSendTransaction.getError();
//        if (error != null) {
//            throw new RuntimeException(error.getMessage());
//        }
//        org.web3j.protocol.core.methods.response.Transaction transaction = web3j
//                .ethGetTransactionByHash(ethSendTransaction.getTransactionHash())
//                .send()
//                .getTransaction()
//                .get();

//        BigInteger gasAmount = transaction.getGas().multiply(transaction.getGasPrice());
//        SResult sResult = new SResult();
//        sResult.setGas(gasAmount.toString());
//        sResult.setTxId(transaction.getHash());

        BigInteger balance = web3j.ethGetBalance(airdropWallet.getAddress(), DefaultBlockParameterName.LATEST).send().getBalance();

        SResult sResult = new SResult();
        sResult.setGas(gasLimit.multiply(gasPrice).toString());
        sResult.setTxId(IdUtil.simpleUUID());
        sResult.setCurrentBalance(balance.toString());
        return sResult;
//        return ethSendTransaction.getTransactionHash();
//        return gasLimit.toString();
    }


}
