package com.ichaoj.ams.service;

import com.baomidou.mybatisplus.extension.service.IService;
import com.ichaoj.ams.entity.AmsAddressAccount;
import com.ichaoj.ams.request.address.BatchAddressRequest;
import com.ichaoj.ams.request.address.PageAddressRequest;
import com.ichaoj.ams.request.address.PremiumAddressRequest;
import com.ichaoj.ams.response.address.AddressResponse;
import com.ichaoj.ams.response.address.CountAddressResponse;
import com.ichaoj.ams.script.model.AirdropWallet;
import com.ichaoj.common.model.PublicPage;

import javax.servlet.http.HttpServletResponse;
import java.util.List;

/**
 * <p>
 * 地址管理 服务类
 * </p>
 *
 * @author cjwen
 * @since 2023-05-10
 */
public interface IAmsAddressAccountService extends IService<AmsAddressAccount> {
    /**
     * 批量创建钱包地址
     *
     * @param batchAddressRequest 请求参数
     * @return 地址列表
     */
    List<String> batchCreateAddress(BatchAddressRequest batchAddressRequest);

    /**
     * 添加精品号
     *
     * @param requestList 精品号地址
     */
    void addPremiumAddress(List<PremiumAddressRequest> requestList);

    /**
     * 批量下载keystore文件
     *
     * @param groupName 地址组名
     * @param response  http 响应
     */
    void batchKeystore(String groupName, HttpServletResponse response);

    /**
     * 判断用户是否创建了地址组
     *
     * @param groupName 地址组名称
     * @param userId    用户id
     * @return 是否创建
     */
    List<AmsAddressAccount> getByGroupNameAndUserId(String groupName, String userId);

    /**
     * 分页查询地址信息
     *
     * @param pageAddressRequest 分页参数
     * @return 分页结果
     */
    PublicPage<AddressResponse> pageAddress(PageAddressRequest pageAddressRequest);

    /**
     * 查询地址组列表
     *
     * @param addressType 地址类型
     * @return 地址组列表
     */
    List<String> queryGroupList(int addressType);

    /**
     * 根据地址查找
     *
     * @param address 地址
     * @return 地址账户
     */
    AmsAddressAccount getByAddress(String address);

    /**
     * 统计用户批量地址的数量（按地址组名称分组）
     *
     * @param userId 用户id
     * @return 统计结果
     */
    List<CountAddressResponse> countBatchCount(String userId);

    /**
     * 查询地址组列表
     *
     * @param addressType 地址类型
     * @return 地址组列表
     */
    List<String> queryBatchAddress(int addressType);

    /**
     * 根据账户账户地址以及私钥
     *
     * @param accountList 地址列表
     * @param userId      用户id
     * @param groupName   地址组名称
     * @param password    密码
     */
    List<AirdropWallet> getAirdropAddressByAccount(List<AmsAddressAccount> accountList, String userId, String groupName, String password);
}
