package com.ichaoj.ams.service;

import com.ichaoj.ams.entity.AmsAddressAccount;
import com.ichaoj.ams.entity.AmsTradeRecord;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ichaoj.ams.request.record.PageTradeRecordRequest;
import com.ichaoj.ams.response.statistics.DailyCostResponse;
import com.ichaoj.ams.response.record.TradeRecordResponse;
import com.ichaoj.ams.response.statistics.ExportResponse;
import com.ichaoj.common.model.PublicPage;

import java.util.List;

/**
 * <p>
 * 交易 服务类
 * </p>
 *
 * @author cjwen
 * @since 2023-05-18
 */
public interface IAmsTradeRecordService extends IService<AmsTradeRecord> {

    /**
     * 分页查询交互记录
     *
     * @param pageRequest 分页参数
     * @return 分页结果
     */
    PublicPage<TradeRecordResponse> pageTradeRecord(PageTradeRecordRequest pageRequest);

    /**
     * 发起随机交易
     *
     * @param accountList 地址列表
     * @param intervalMin 时间间隔最小分钟数
     * @param intervalMax 时间间隔最大分钟数
     * @param amount      交易金额
     * @param maxGas      最大gas
     * @param executeId   执行id
     */
    void randomTrans(
            List<AmsAddressAccount> accountList,
            Integer intervalMin,
            Integer intervalMax,
            String amount,
            String maxGas,
            String executeId);

    /**
     * 统计每日消耗gas和本金
     * @return 统计数据
     */
    List<DailyCostResponse> dailyCostStatistics();

    /**
     * 获取导出数据
     * @return 导出的数据
     */
    List<ExportResponse> getExportData();

    /**
     * 根据用户id查询用户投入列表
     * @param userId 用户
     * @return 列表
     */
    List<TradeRecordResponse> listByUserId(String userId);
}
