package com.ichaoj.ams.service.impl;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.collection.CollectionUtil;
import cn.hutool.core.date.DateTime;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.date.LocalDateTimeUtil;
import cn.hutool.core.thread.ThreadUtil;
import cn.hutool.core.util.IdUtil;
import cn.hutool.core.util.RandomUtil;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import com.ichaoj.ams.entity.AmsAddressAccount;
import com.ichaoj.ams.entity.AmsExecuteRecord;
import com.ichaoj.ams.entity.AmsTradeRecord;
import com.ichaoj.ams.mapper.AmsTradeRecordMapper;
import com.ichaoj.ams.request.record.CreateTradeRecordRequest;
import com.ichaoj.ams.request.record.PageTradeRecordRequest;
import com.ichaoj.ams.response.statistics.DailyCostResponse;
import com.ichaoj.ams.response.record.TradeRecordResponse;
import com.ichaoj.ams.response.statistics.ExportResponse;
import com.ichaoj.ams.service.IAmsAddressAccountService;
import com.ichaoj.ams.service.IAmsTradeRecordService;
import com.ichaoj.common.model.PublicPage;
import com.ichaoj.common.model.PublicUserInfo;
import com.ichaoj.mybatis.service.SuperWhaleServiceImpl;
import com.ichaoj.web.context.SuperWhaleContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.context.annotation.Lazy;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.math.BigDecimal;
import java.time.LocalDateTime;
import java.util.*;
import java.util.concurrent.*;
import java.util.stream.Collectors;

/**
 * <p>
 * 交易 服务实现类
 * </p>
 *
 * @author cjwen
 * @since 2023-05-18
 */
@Service
@Slf4j
public class AmsTradeRecordServiceImpl extends SuperWhaleServiceImpl<AmsTradeRecordMapper, AmsTradeRecord> implements IAmsTradeRecordService {

//    private final static ScheduledExecutorService AMS_SCHEDULER = Executors.newScheduledThreadPool(5);

    @Resource
    @Lazy
    private IAmsAddressAccountService accountService;

    @Override
    public PublicPage<TradeRecordResponse> pageTradeRecord(PageTradeRecordRequest pageRequest) {
        Page<TradeRecordResponse> result = this.baseMapper.pageTradeRecord(this.buildPageObj(pageRequest), pageRequest);
        return this.convertPublicPage(result, s -> BeanUtil.copyProperties(s, TradeRecordResponse.class));
    }

    @Override
    public void randomTrans(List<AmsAddressAccount> accountList, Integer intervalMin, Integer intervalMax, String amount, String maxGas, String executeId) {
        Map<String, Boolean> addressMap = new HashMap<>(accountList.size());
        Set<String> set = new HashSet<>();

        List<String> addresses = accountList.stream().map(AmsAddressAccount::getAddress).collect(Collectors.toList());

        for (String address : addresses) {
            addressMap.put(address, false);
        }
        prepareTrans(intervalMin, intervalMax, amount, maxGas, executeId, addressMap, set, addresses);
    }

    @Transactional(rollbackFor = Exception.class)
    protected synchronized void prepareTrans(Integer intervalMin, Integer intervalMax, String amount, String maxGas, String executeId, Map<String, Boolean> addressMap, Set<String> set, List<String> addresses) {
        int i = 0;
        while (set.size() < addresses.size()) {
            BigDecimal gas = getCurrentGasPrice(maxGas);
            BigDecimal currenAmount = getCurrenAmount(amount);
            String address = getUnusedAddress(addressMap, addresses);

            AmsTradeRecord record = this.getOne(
                    new LambdaQueryWrapper<AmsTradeRecord>()
                            .eq(AmsTradeRecord::getExecuteId, executeId)
                            .eq(AmsTradeRecord::getAddress, address)
            );
            if (record != null) {
                break;
            }
            addressMap.put(address, true);
            set.add(address);
            int sleepSeconds = RandomUtil.randomInt(intervalMin, intervalMax);
            long sleep = sleepSeconds * 60L + RandomUtil.randomInt(1, 30);
            if (i == 0) {
                log.info("执行器{} ，当前总共有{} 条交易需要执行，当前是第{} 笔交易，第{}笔交易需要等待{} 分钟",
                        executeId,
                        addresses.size(),
                        i + 1,
                        i + 2,
                        sleep / 60);
                log.info("当前gas: {}", gas);
                log.info("amount: {}", currenAmount);
                executeTrans(executeId, address, gas, currenAmount);
                ThreadUtil.safeSleep(sleep);
                i++;
            } else {
                AmsTradeRecord tradeRecord = this.getOne(
                        new LambdaQueryWrapper<AmsTradeRecord>()
                                .eq(AmsTradeRecord::getExecuteId, executeId)
                                .eq(AmsTradeRecord::getAddress, address)
                );

                if (tradeRecord == null) {
                    log.info("执行器{} ，当前总共有{} 条交易需要执行，当前是第{} 笔交易，第{}笔交易需要等待{} 分钟",
                            executeId,
                            addresses.size(),
                            i + 1,
                            i + 2,
                            sleep / 60);
                    log.info("当前gas: {}", gas);
                    log.info("amount: {}", currenAmount);
//                    AMS_SCHEDULER.schedule(() -> {
//                        // todo 调用链上交易
//
//                        //交易入库
//                        executeTrans(executeId, address, gas, currenAmount);
//
//                    }, sleep, TimeUnit.SECONDS);
                    i++;
                }
            }
        }
    }

    private void executeTrans(String executeId, String address, BigDecimal gas, BigDecimal currenAmount) {
        // todo 调用链上交易
        //交易入库
        String txId = saveTrade(executeId, address, gas, currenAmount);

        // 修改地址操作时间
        AmsAddressAccount account = accountService.getByAddress(address);
        account.setLastOperTime(LocalDateTime.now());
        accountService.updateById(account);

        // 修改当前余额
        AmsTradeRecord record = this.getById(txId);
        record.setCurrentBalance(getCurrentBalance(address));
        this.updateById(record);
    }

    @Override
    public List<DailyCostResponse> dailyCostStatistics() {
        return this.baseMapper.dailyCostStatistics();
    }

    @Override
    public List<ExportResponse> getExportData() {
        String userId = SuperWhaleContext.getContext(PublicUserInfo.class).getUserId();
        return this.baseMapper.getExportData(userId);
    }

    @Override
    public List<TradeRecordResponse> listByUserId(String userId) {
        return this.baseMapper.listByUserId(userId);
    }

    private BigDecimal getCurrenAmount(String amount) {
        return RandomUtil.randomBigDecimal(BigDecimal.ZERO, new BigDecimal(amount));
    }

    private BigDecimal getCurrentGasPrice(String maxGas) {
        return RandomUtil.randomBigDecimal(BigDecimal.ZERO, new BigDecimal(maxGas));
    }

    private String saveTrade(String executeId, String address, BigDecimal gas, BigDecimal amount) {
        CreateTradeRecordRequest trade = CreateTradeRecordRequest.builder()
                .address(address)
                .executeId(executeId)
                .createTime(LocalDateTime.now())
                .status(0)
                .txId(IdUtil.simpleUUID())
                .gas(gas.toPlainString())
                .amount(amount.toPlainString())
                .build();
        AmsTradeRecord record = BeanUtil.copyProperties(trade, AmsTradeRecord.class);
        this.save(record);
        return record.getTradeRecordId();
    }

    private String getCurrentBalance(String address) {
        // todo 查询链上余额
        return "100";
    }

    private String getUnusedAddress(Map<String, Boolean> addressMap, List<String> addresses) {
        String address = null;
        while (address == null) {
            address = getRandomAddress(addresses);
            if (addressMap.get(address)) {
                address = null;
            }
        }
        return address;
    }

    private String getRandomAddress(List<String> accountList) {
        int randomInt = RandomUtil.randomInt(0, accountList.size());
        return accountList.get(randomInt);
    }


    @Scheduled(cron = "0/59 * * * * ? ")
    public void scanTradeStatus() {
        List<AmsTradeRecord> list = this.list(new LambdaQueryWrapper<AmsTradeRecord>()
                .eq(AmsTradeRecord::getStatus, 0));
        for (AmsTradeRecord record : list) {
            DateTime dateTime = DateUtil.offsetMinute(DateUtil.date(record.getCreateTime()), 1);
            if (new DateTime().isAfterOrEquals(dateTime)) {
                record.setStatus(1);
                this.updateById(record);
            }
        }
    }

}
